package org.seasar.extension.dataset.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

import org.seasar.extension.dataset.DataTable;
import org.seasar.extension.dataset.impl.SqlTableWriter;
import org.seasar.extension.jdbc.util.ConnectionUtil;
import org.seasar.extension.jdbc.util.DataSourceUtil;
import org.seasar.framework.exception.SQLRuntimeException;
import org.seasar.framework.log.Logger;
import org.seasar.framework.util.ResultSetUtil;
import org.seasar.framework.util.StatementUtil;

public class SqlServerSqlTableWriter extends SqlTableWriter {

	private static final Logger LOGGER = Logger.getLogger(SqlTableWriter.class);

	private static final Map HAS_IDENTITY_COLUMN = Collections
			.synchronizedMap(new HashMap());

	public SqlServerSqlTableWriter(final DataSource dataSource) {
		super(dataSource);
	}

	protected void doWrite(final DataTable dataTable) {
		turnIdentityInsert(dataTable, true);
		super.doWrite(dataTable);
		turnIdentityInsert(dataTable, false);
	}

	private void turnIdentityInsert(final DataTable dataTable,
			final boolean identityInsert) {
		final Connection connection = DataSourceUtil
				.getConnection(getDataSource());
		try {
			if (hasIdentityColumn(connection, dataTable)) {
				final String sql = "SET IDENTITY_INSERT "
						+ dataTable.getTableName() + " "
						+ (identityInsert ? "ON" : "OFF");
				if (LOGGER.isDebugEnabled()) {
					LOGGER.debug(sql);
				}
				execute(connection, sql);
			}
		} finally {
			ConnectionUtil.close(connection);
		}
	}

	private boolean hasIdentityColumn(final Connection connection,
			final DataTable dataTable) {
		final String tableName = dataTable.getTableName();
		if (!HAS_IDENTITY_COLUMN.containsKey(tableName)) {
			final String sql = "SELECT IDENT_CURRENT ('"
					+ dataTable.getTableName() + "') AS IDENT_CURRENT";
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug(sql);
			}
			Object identCurrent = queryToOneColumn(connection, sql);
			HAS_IDENTITY_COLUMN.put(tableName, Boolean
					.valueOf(identCurrent != null));
		}
		return ((Boolean) HAS_IDENTITY_COLUMN.get(tableName)).booleanValue();
	}

	private boolean execute(final Connection connection, final String sql) {
		Statement statement = null;
		try {
			statement = connection.createStatement();
			return statement.execute(sql);
		} catch (final SQLException e) {
			throw new SQLRuntimeException(e);
		} finally {
			StatementUtil.close(statement);
		}
	}

	private Object queryToOneColumn(final Connection connection,
			final String sql) {
		final PreparedStatement statement = ConnectionUtil.prepareStatement(
				connection, sql);
		ResultSet resultSet = null;
		try {
			resultSet = statement.executeQuery();
			Object column = null;
			while (resultSet.next()) {
				column = resultSet.getObject(1);
			}
			return column;
		} catch (final SQLException e) {
			throw new SQLRuntimeException(e);
		} finally {
			ResultSetUtil.close(resultSet);
			StatementUtil.close(statement);
		}
	}
}
